@@ -230,9 +230,6 @@ class ResponseSchema(api.Api):
                                 'uri': {
                                     'type': 'string'
                                 },
-                                'group': {
-                                    'type': ['string', 'null']
-                                },
                                 'flavor': {
                                     'type': ['string', 'null']
                                 },
@@ -241,7 +238,7 @@ class ResponseSchema(api.Api):
                                     'additionalProperties': True
                                 }
                             },
-                            'required': ['href', 'weight', 'uri', 'group'],
+                            'required': ['href', 'weight', 'uri'],
                             'additionalProperties': False,
                         },
                     }
@@ -288,9 +285,6 @@ class ResponseSchema(api.Api):
                     'uri': {
                         'type': 'string'
                     },
-                    'group': {
-                        'type': ['string', 'null']
-                    },
                     'flavor': {
                         'type': ['string', 'null']
                     },
