@@ -24,6 +24,7 @@ Schema:
 """
 
 import functools
+from oslo_log import log as logging
 from pymongo import errors as mongo_error
 
 from zaqar.common import utils as common_utils
@@ -35,6 +36,8 @@ POOLS_INDEX = [
     ('n', 1)
 ]
 
+LOG = logging.getLogger(__name__)
+
 URI_INDEX = [
     ('u', 1)
 ]
@@ -95,8 +98,6 @@ class PoolsController(base.PoolsBase):
         query = None
         if flavor is None:
             query = {'f': None}
-        elif flavor.get("pool_group") is not None:
-            query = {'g': flavor.get("pool_group")}
         elif flavor.get('name') is not None:
             query = {'f': flavor.get('name')}
         cursor = self._col.find(query,
@@ -105,7 +106,7 @@ class PoolsController(base.PoolsBase):
         return utils.HookedCursor(cursor, normalizer)
 
     @utils.raises_conn_error
-    def _create(self, name, weight, uri, group=None, flavor=None,
+    def _create(self, name, weight, uri, flavor=None,
                 options=None):
         options = {} if options is None else options
         try:
@@ -113,11 +114,11 @@ class PoolsController(base.PoolsBase):
                                  {'$set': {'n': name,
                                            'w': weight,
                                            'u': uri,
-                                           'g': group,
                                            'f': flavor,
                                            'o': options}},
                                  upsert=True)
         except mongo_error.DuplicateKeyError:
+            LOG.exception('Pool "%s" already exists', name)
             raise errors.PoolAlreadyExists()
 
     @utils.raises_conn_error
@@ -126,19 +127,16 @@ class PoolsController(base.PoolsBase):
 
     @utils.raises_conn_error
     def _update(self, name, **kwargs):
-        names = ('uri', 'weight', 'group', 'flavor', 'options')
+        names = ('uri', 'weight', 'flavor', 'options')
         fields = common_utils.fields(kwargs, names,
                                      pred=lambda x: x is not None,
                                      key_transform=lambda x: x[0])
-        assert fields, ('`weight`, `uri`, `group`, '
+        assert fields, ('`weight`, `uri`, '
                         'or `options` not found in kwargs')
 
         flavor = fields.get('f')
         if flavor is not None and len(flavor) == 0:
             fields['f'] = None
-        group = fields.get('g')
-        if group is not None and len(group) == 0:
-            fields['g'] = None
 
         res = self._col.update_one({'n': name},
                                    {'$set': fields},
@@ -153,20 +151,6 @@ class PoolsController(base.PoolsBase):
         # recursion error.
         try:
             pool = self.get(name)
-            if pool['group'] is not None:
-                flavor = {}
-                flavor['pool_group'] = pool['group']
-                pools_group = self.get_pools_by_flavor(flavor=flavor)
-                flavor_ctl = self.driver.flavors_controller
-                res = list(flavor_ctl._list_by_pool_group(pool['group']))
-
-                # NOTE(flaper87): If this is the only pool in the
-                # group and it's being used by a flavor, don't allow
-                # it to be deleted.
-                if res and len(pools_group) == 1:
-                    flavors = ', '.join([x['name'] for x in res])
-                    raise errors.PoolInUseByFlavor(name, flavors)
-
             pools_in_flavor = []
             flavor = pool.get("flavor", None)
             if flavor is not None:
@@ -191,7 +175,6 @@ class PoolsController(base.PoolsBase):
 def _normalize(pool, detailed=False):
     ret = {
         'name': pool['n'],
-        'group': pool['g'],
         'flavor': pool['f'],
         'uri': pool['u'],
         'weight': pool['w'],
