@@ -162,8 +162,8 @@ class QueueController(storage.Queue):
                     projection={'c.v': 1, '_id': 0})
 
                 break
-            except pymongo.errors.AutoReconnect as ex:
-                LOG.exception(ex)
+            except pymongo.errors.AutoReconnect:
+                LOG.exception('Auto reconnect failure')
 
         if doc is None:
             if window is None:
@@ -196,10 +196,11 @@ class QueueController(storage.Queue):
         except errors.QueueDoesNotExist:
             return {}
 
-    def _list(self, project=None, marker=None,
-              limit=storage.DEFAULT_QUEUES_PER_PAGE, detailed=False):
+    def _list(self, project=None, kfilter={}, marker=None,
+              limit=storage.DEFAULT_QUEUES_PER_PAGE, detailed=False,
+              name=None):
 
-        query = utils.scoped_query(marker, project)
+        query = utils.scoped_query(marker, project, name, kfilter)
 
         projection = {'p_q': 1, '_id': 0}
         if detailed:
@@ -286,6 +287,13 @@ class QueueController(storage.Queue):
     def _stats(self, name, project=None):
         pass
 
+    @utils.raises_conn_error
+    @utils.retries_on_autoreconnect
+    def _calculate_resource_count(self, project=None):
+        query = utils.scoped_query(None, project, None, {})
+        projection = {'p_q': 1, '_id': 0}
+        return self._collection.find(query, projection=projection).count()
+
 
 def _get_scoped_query(name, project):
     return {'p_q': utils.scope_queue_name(name, project)}
