@@ -135,7 +135,7 @@ class MessageController(storage.Message, scripting.Mixin):
         return self._client.zcard(utils.msgset_key(queue, project))
 
     def _create_msgset(self, queue, project, pipe):
-        pipe.zadd(MSGSET_INDEX_KEY, 1, utils.msgset_key(queue, project))
+        pipe.zadd(MSGSET_INDEX_KEY, {utils.msgset_key(queue, project): 1})
 
     def _delete_msgset(self, queue, project, pipe):
         pipe.zrem(MSGSET_INDEX_KEY, utils.msgset_key(queue, project))
@@ -243,7 +243,10 @@ class MessageController(storage.Message, scripting.Mixin):
             # of the queue; otherwise we would just filter them all
             # out and likely end up with an empty list to return.
             marker = self._find_first_unclaimed(queue, project, limit)
-            start = client.zrank(msgset_key, marker) or 0
+            if marker:
+                start = client.zrank(msgset_key, marker) or 0
+            else:
+                start = 0
         else:
             rank = client.zrank(msgset_key, marker)
             start = rank + 1 if rank else 0
@@ -500,7 +503,7 @@ class MessageController(storage.Message, scripting.Mixin):
 
     @utils.raises_conn_error
     @utils.retries_on_connection_error
-    def bulk_delete(self, queue, message_ids, project=None):
+    def bulk_delete(self, queue, message_ids, project=None, claim_ids=None):
         claim_ctrl = self.driver.claim_controller
         if not self._queue_ctrl.exists(queue, project):
             return
@@ -516,7 +519,14 @@ class MessageController(storage.Message, scripting.Mixin):
                 pipe.zrem(msgset_key, mid)
 
                 msg_claim = self._get_claim(mid)
+
+                if claim_ids and msg_claim is None:
+                    raise errors.MessageNotClaimed(mid)
+
                 if msg_claim is not None:
+                    if claim_ids and (msg_claim['id'] not in claim_ids):
+                        raise errors.ClaimDoesNotMatch(msg_claim['id'], queue,
+                                                       project)
                     claim_ctrl._del_message(queue, project, msg_claim['id'],
                                             mid, pipe)
             pipe.execute()
@@ -545,7 +555,7 @@ def _filter_messages(messages, filters, to_basic, marker):
     """Create a filtering iterator over a list of messages.
 
     The function accepts a list of filters to be filtered
-    before the the message can be included as a part of the reply.
+    before the message can be included as a part of the reply.
     """
     now = timeutils.utcnow_ts()
 
@@ -627,3 +637,53 @@ class MessageQueueHandler(object):
                 message_stats['oldest'] = oldest
 
         return {'messages': message_stats}
+
+
+class MessageTopicHandler(object):
+    def __init__(self, driver, control_driver):
+        self.driver = driver
+        self._client = self.driver.connection
+        self._topic_ctrl = self.driver.topic_controller
+        self._message_ctrl = self.driver.message_controller
+
+    @utils.raises_conn_error
+    def create(self, name, metadata=None, project=None):
+        with self._client.pipeline() as pipe:
+            self._message_ctrl._create_msgset(name, project, pipe)
+
+            try:
+                pipe.execute()
+            except redis.exceptions.ResponseError:
+                return False
+
+    @utils.raises_conn_error
+    @utils.retries_on_connection_error
+    def delete(self, name, project=None):
+        with self._client.pipeline() as pipe:
+            self._message_ctrl._delete_msgset(name, project, pipe)
+            self._message_ctrl._delete_queue_messages(name, project, pipe)
+            pipe.execute()
+
+    @utils.raises_conn_error
+    @utils.retries_on_connection_error
+    def stats(self, name, project=None):
+        if not self._topic_ctrl.exists(name, project=project):
+            raise errors.TopicDoesNotExist(name, project)
+
+        total = self._message_ctrl._count(name, project)
+
+        message_stats = {
+            'total': total
+        }
+
+        if total:
+            try:
+                newest = self._message_ctrl.first(name, project, -1)
+                oldest = self._message_ctrl.first(name, project, 1)
+            except errors.QueueIsEmpty:
+                pass
+            else:
+                message_stats['newest'] = newest
+                message_stats['oldest'] = oldest
+
+        return {'messages': message_stats}
