@@ -95,7 +95,7 @@ def load_storage_impl(uri, control_mode=False, default_store=None):
         return mgr.driver
 
     except Exception as exc:
-        LOG.exception(exc)
+        LOG.exception('Error loading storage driver')
         raise errors.InvalidDriver(exc)
 
 
@@ -128,9 +128,9 @@ def load_storage_driver(conf, cache, storage_type=None,
 
     driver_type = 'zaqar.{0}.storage'.format(mode)
 
-    _invoke_args = [conf, cache]
+    _invoke_args = (conf, cache)
     if control_driver is not None:
-        _invoke_args.append(control_driver)
+        _invoke_args = (conf, cache, control_driver)
 
     try:
         mgr = driver.DriverManager(driver_type,
@@ -148,9 +148,8 @@ def load_storage_driver(conf, cache, storage_type=None,
             return mgr.driver
 
     except Exception as exc:
-        LOG.error('Failed to load "{}" driver for "{}"'.format(
-            driver_type, storage_type))
-        LOG.exception(exc)
+        LOG.exception('Failed to load "%s" driver for "%s"',
+                      driver_type, storage_type)
         raise errors.InvalidDriver(exc)
 
 
