@@ -35,7 +35,6 @@ from oslo_config import cfg
 from oslo_db.sqlalchemy import test_migrations as t_m
 from oslo_log import log as logging
 
-from zaqar.i18n import _LE
 import zaqar.storage.sqlalchemy.migration
 from zaqar.storage.sqlalchemy import tables
 
@@ -71,8 +70,7 @@ class BaseWalkMigrationTestCase(object):
         """
 
         CONF.set_override('uri', str(engine.url),
-                          group='drivers:management_store:sqlalchemy',
-                          enforce_type=True)
+                          group='drivers:management_store:sqlalchemy')
 
     def _alembic_command(self, alembic_command, engine, *args, **kwargs):
         """Most of alembic command return data into output.
@@ -81,8 +79,7 @@ class BaseWalkMigrationTestCase(object):
         """
         self.ALEMBIC_CONFIG.stdout = buf = io.StringIO()
         CONF.set_override('uri', str(engine.url),
-                          group='drivers:management_store:sqlalchemy',
-                          enforce_type=True)
+                          group='drivers:management_store:sqlalchemy')
 
         getattr(command, alembic_command)(*args, **kwargs)
         res = buf.getvalue().strip()
@@ -162,8 +159,8 @@ class BaseWalkMigrationTestCase(object):
                 if check:
                     check(engine, data)
         except Exception:
-            LOG.error(_LE("Failed to migrate to version {version} on engine "
-                      "{engine}").format(version=version, engine=engine))
+            LOG.error("Failed to migrate to version {version} on engine "
+                      "{engine}".format(version=version, engine=engine))
             raise
 
 
@@ -184,8 +181,7 @@ class TestModelsMigrationsSync(t_m.ModelsMigrationsSync):
 
     def db_sync(self, engine):
         CONF.set_override('uri', str(engine.url),
-                          group='drivers:management_store:sqlalchemy',
-                          enforce_type=True)
+                          group='drivers:management_store:sqlalchemy')
         script_location = os.path.join(self.mg_path, 'alembic_migrations')
         self.ALEMBIC_CONFIG.set_main_option('script_location', script_location)
         alembic.command.upgrade(self.ALEMBIC_CONFIG, 'head')
