@@ -47,8 +47,8 @@ class SubscriptionTest(base.V1_1Base):
                                         body, self.headers)
 
         def validator(resp, isBinary):
-            resp = json.loads(resp)
-            self.assertEqual(resp['headers']['status'], 201)
+            resp = json.loads(resp.decode())
+            self.assertIn(resp['headers']['status'], [201, 204])
 
         with mock.patch.object(self.protocol, 'sendMessage') as msg_mock:
             msg_mock.side_effect = validator
@@ -66,7 +66,7 @@ class SubscriptionTest(base.V1_1Base):
                                         body, self.headers)
 
         def validator(resp, isBinary):
-            resp = json.loads(resp)
+            resp = json.loads(resp.decode())
             self.assertEqual(resp['headers']['status'], 204)
 
         sender.side_effect = validator
@@ -110,7 +110,7 @@ class SubscriptionTest(base.V1_1Base):
                         'api': 'v2', 'headers': self.headers}}
 
         self.assertEqual(1, sender.call_count)
-        self.assertEqual(response, json.loads(sender.call_args[0][0]))
+        self.assertEqual(response, json.loads(sender.call_args[0][0].decode()))
 
         # Trigger protocol close
         self.protocol.onClose(True, 100, None)
@@ -180,7 +180,7 @@ class SubscriptionTest(base.V1_1Base):
                                  'subscription_id': str(sub)},
                         'api': 'v2', 'headers': self.headers}}
         self.assertEqual(1, sender.call_count)
-        self.assertEqual(response, json.loads(sender.call_args[0][0]))
+        self.assertEqual(response, json.loads(sender.call_args[0][0].decode()))
 
     def test_subscription_create_no_queue(self):
         action = consts.SUBSCRIPTION_CREATE
@@ -214,7 +214,7 @@ class SubscriptionTest(base.V1_1Base):
                         'api': 'v2', 'headers': self.headers}}
 
         self.assertEqual(1, sender.call_count)
-        self.assertEqual(response, json.loads(sender.call_args[0][0]))
+        self.assertEqual(response, json.loads(sender.call_args[0][0].decode()))
 
     def test_subscription_get(self):
         sub = self.boot.storage.subscription_controller.create(
@@ -246,7 +246,7 @@ class SubscriptionTest(base.V1_1Base):
                         'api': 'v2', 'headers': self.headers}}
 
         self.assertEqual(1, sender.call_count)
-        response = json.loads(sender.call_args[0][0])
+        response = json.loads(sender.call_args[0][0].decode())
         # Get and remove age from the actual response.
         actual_sub_age = response['body'].pop('age')
         self.assertLessEqual(0, actual_sub_age)
@@ -282,7 +282,7 @@ class SubscriptionTest(base.V1_1Base):
                         'body': {'queue_name': 'kitkat'},
                         'api': 'v2', 'headers': self.headers}}
         self.assertEqual(1, sender.call_count)
-        response = json.loads(sender.call_args[0][0])
+        response = json.loads(sender.call_args[0][0].decode())
         # Get and remove age from the actual response.
         actual_sub_age = response['body']['subscriptions'][0].pop('age')
         self.assertLessEqual(0, actual_sub_age)
@@ -334,7 +334,8 @@ class SubscriptionTest(base.V1_1Base):
 
         # Check that the server responded in text format to the message
         # creation request
-        message_create_response = json.loads(sender.call_args_list[1][0][0])
+        message_create_response = json.loads(
+            sender.call_args_list[1][0][0].decode())
         self.assertEqual(201, message_create_response['headers']['status'])
 
         # Fetch webhook notification that was intended to arrive to
@@ -368,7 +369,7 @@ class SubscriptionTest(base.V1_1Base):
         req = test_utils.create_request(action, body, self.headers)
 
         def validator(resp, isBinary):
-            resp = json.loads(resp)
+            resp = json.loads(resp.decode())
             self.assertEqual(503, resp['headers']['status'])
 
         sender.side_effect = validator
