@@ -110,6 +110,57 @@ class TestQueueLifecycleMongoDB(base.V2Base):
         ref_doc['_default_message_ttl'] = 3600
         ref_doc['_max_messages_post_size'] = 262144
         ref_doc['_default_message_delay'] = 0
+        ref_doc['_dead_letter_queue'] = None
+        ref_doc['_dead_letter_queue_messages_ttl'] = None
+        ref_doc['_max_claim_count'] = None
+        self.assertEqual(ref_doc, result_doc)
+
+        # Stats empty queue
+        self.simulate_get(gumshoe_queue_path_stats, headers=headers)
+        self.assertEqual(falcon.HTTP_200, self.srmock.status)
+
+        # Delete
+        self.simulate_delete(self.gumshoe_queue_path, headers=headers)
+        self.assertEqual(falcon.HTTP_204, self.srmock.status)
+
+        # Get non-existent stats
+        self.simulate_get(gumshoe_queue_path_stats, headers=headers)
+        self.assertEqual(falcon.HTTP_200, self.srmock.status)
+
+    @ddt.data('1234567890', '11111111111111111111111111111111111')
+    def test_basics_thoroughly_with_different_client_id(self, client_id):
+        self.conf.set_override('client_id_uuid_safe', 'off', 'transport')
+        headers = {
+            'Client-ID': client_id,
+            'X-Project-ID': '480924'
+        }
+        gumshoe_queue_path_stats = self.gumshoe_queue_path + '/stats'
+
+        # Stats are empty - queue not created yet
+        self.simulate_get(gumshoe_queue_path_stats, headers=headers)
+        self.assertEqual(falcon.HTTP_200, self.srmock.status)
+
+        # Create
+        doc = '{"messages": {"ttl": 600}}'
+        self.simulate_put(self.gumshoe_queue_path,
+                          headers=headers, body=doc)
+        self.assertEqual(falcon.HTTP_201, self.srmock.status)
+
+        location = self.srmock.headers_dict['Location']
+        self.assertEqual(location, self.gumshoe_queue_path)
+
+        # Fetch metadata
+        result = self.simulate_get(self.gumshoe_queue_path,
+                                   headers=headers)
+        result_doc = jsonutils.loads(result[0])
+        self.assertEqual(falcon.HTTP_200, self.srmock.status)
+        ref_doc = jsonutils.loads(doc)
+        ref_doc['_default_message_ttl'] = 3600
+        ref_doc['_max_messages_post_size'] = 262144
+        ref_doc['_default_message_delay'] = 0
+        ref_doc['_dead_letter_queue'] = None
+        ref_doc['_dead_letter_queue_messages_ttl'] = None
+        ref_doc['_max_claim_count'] = None
         self.assertEqual(ref_doc, result_doc)
 
         # Stats empty queue
@@ -247,6 +298,9 @@ class TestQueueLifecycleMongoDB(base.V2Base):
         ref_doc['_default_message_ttl'] = 3600
         ref_doc['_max_messages_post_size'] = 262144
         ref_doc['_default_message_delay'] = 0
+        ref_doc['_dead_letter_queue'] = None
+        ref_doc['_dead_letter_queue_messages_ttl'] = None
+        ref_doc['_max_claim_count'] = None
         self.assertEqual(ref_doc, result_doc)
         self.assertEqual(falcon.HTTP_200, self.srmock.status)
 
@@ -301,7 +355,10 @@ class TestQueueLifecycleMongoDB(base.V2Base):
         self.assertEqual({'key1': 2, 'key2': 1,
                           '_default_message_ttl': 300,
                           '_max_messages_post_size': 262144,
-                          '_default_message_delay': 0}, result_doc)
+                          '_default_message_delay': 0,
+                          '_dead_letter_queue': None,
+                          '_dead_letter_queue_messages_ttl': None,
+                          '_max_claim_count': None}, result_doc)
 
         # remove metadata
         doc3 = '[{"op":"remove", "path": "/metadata/key1"}]'
@@ -323,7 +380,10 @@ class TestQueueLifecycleMongoDB(base.V2Base):
         result_doc = jsonutils.loads(result[0])
         self.assertEqual({'key2': 1, '_default_message_ttl': 3600,
                           '_max_messages_post_size': 262144,
-                          '_default_message_delay': 0}, result_doc)
+                          '_default_message_delay': 0,
+                          '_dead_letter_queue': None,
+                          '_dead_letter_queue_messages_ttl': None,
+                          '_max_claim_count': None}, result_doc)
 
         # replace non-existent metadata
         doc4 = '[{"op":"replace", "path": "/metadata/key3", "value":2}]'
@@ -417,7 +477,7 @@ class TestQueueLifecycleMongoDB(base.V2Base):
 
         result_doc = jsonutils.loads(result[0])
         [target, params] = result_doc['links'][0]['href'].split('?')
-
+        self.simulate_get(target, headers=header, query_string=params)
         self.assertEqual(falcon.HTTP_200, self.srmock.status)
 
         # Ensure we didn't pick up the queue from the alt project.
@@ -438,7 +498,17 @@ class TestQueueLifecycleMongoDB(base.V2Base):
         self.assertEqual(queue['metadata'], result_doc)
         self.assertEqual({'node': 31, '_default_message_ttl': 3600,
                           '_max_messages_post_size': 262144,
-                          '_default_message_delay': 0},  result_doc)
+                          '_default_message_delay': 0,
+                          '_dead_letter_queue': None,
+                          '_dead_letter_queue_messages_ttl': None,
+                          '_max_claim_count': None},  result_doc)
+
+        # queue filter
+        result = self.simulate_get(self.queue_path, headers=header,
+                                   query_string='node=34')
+        self.assertEqual(falcon.HTTP_200, self.srmock.status)
+        result_doc = jsonutils.loads(result[0])
+        self.assertEqual(0, len(result_doc['queues']))
 
         # List tail
         self.simulate_get(target, headers=header, query_string=params)
@@ -473,6 +543,57 @@ class TestQueueLifecycleMongoDB(base.V2Base):
             self.simulate_get(self.queue_path, headers=header)
             self.assertEqual(falcon.HTTP_503, self.srmock.status)
 
+    def test_list_with_filter(self):
+        arbitrary_number = 644079696574693
+        project_id = str(arbitrary_number)
+        client_id = uuidutils.generate_uuid()
+        header = {
+            'X-Project-ID': project_id,
+            'Client-ID': client_id
+        }
+
+        # Create some
+        def create_queue(name, project_id, body):
+            altheader = {'Client-ID': client_id}
+            if project_id is not None:
+                altheader['X-Project-ID'] = project_id
+            uri = self.queue_path + '/' + name
+            self.simulate_put(uri, headers=altheader, body=body)
+
+        create_queue('q1', project_id, '{"test_metadata_key1": "value1"}')
+        create_queue('q2', project_id, '{"_max_messages_post_size": 2000}')
+        create_queue('q3', project_id, '{"test_metadata_key2": 30}')
+
+        # List (filter query)
+        result = self.simulate_get(self.queue_path, headers=header,
+                                   query_string='name=q&test_metadata_key2=30')
+
+        result_doc = jsonutils.loads(result[0])
+        self.assertEqual(1, len(result_doc['queues']))
+        self.assertEqual('q3', result_doc['queues'][0]['name'])
+
+        # List (filter query)
+        result = self.simulate_get(self.queue_path, headers=header,
+                                   query_string='_max_messages_post_size=2000')
+
+        result_doc = jsonutils.loads(result[0])
+        self.assertEqual(1, len(result_doc['queues']))
+        self.assertEqual('q2', result_doc['queues'][0]['name'])
+
+        # List (filter query)
+        result = self.simulate_get(self.queue_path, headers=header,
+                                   query_string='name=q')
+
+        result_doc = jsonutils.loads(result[0])
+        self.assertEqual(3, len(result_doc['queues']))
+
+        # List (filter query)
+        result = self.simulate_get(self.queue_path, headers=header,
+                                   query_string='with_count=true')
+
+        result_doc = jsonutils.loads(result[0])
+        self.assertEqual(3, result_doc['count'])
+
 
 class TestQueueLifecycleFaultyDriver(base.V2BaseFaulty):
 
