@@ -28,17 +28,30 @@ from zaqar.transport.wsgi import utils as wsgi_utils
 
 LOG = logging.getLogger(__name__)
 
-MESSAGE_POST_SPEC = (('ttl', int, None), ('body', '*', None))
-
 
 class CollectionResource(object):
 
-    __slots__ = ('_message_controller', '_wsgi_conf', '_validate')
+    __slots__ = (
+        '_message_controller',
+        '_queue_controller',
+        '_wsgi_conf',
+        '_validate',
+        '_message_post_spec',
+    )
+
+    def __init__(self, wsgi_conf, validate,
+                 message_controller, queue_controller,
+                 default_message_ttl):
 
-    def __init__(self, wsgi_conf, validate, message_controller):
         self._wsgi_conf = wsgi_conf
         self._validate = validate
         self._message_controller = message_controller
+        self._queue_controller = queue_controller
+
+        self._message_post_spec = (
+            ('ttl', int, default_message_ttl),
+            ('body', '*', None),
+        )
 
     # ----------------------------------------------------------------------
     # Helpers
@@ -57,9 +70,9 @@ class CollectionResource(object):
             LOG.debug(ex)
             raise wsgi_errors.HTTPBadRequestAPI(six.text_type(ex))
 
-        except Exception as ex:
-            LOG.exception(ex)
+        except Exception:
             description = _(u'Message could not be retrieved.')
+            LOG.exception(description)
             raise wsgi_errors.HTTPServiceUnavailable(description)
 
         # Prepare response
@@ -67,7 +80,10 @@ class CollectionResource(object):
         if not messages:
             return None
 
-        return [wsgi_utils.format_message_v1(m, base_path) for m in messages]
+        messages = [wsgi_utils.format_message_v1_1(m, base_path, m['claim_id'])
+                    for m in messages]
+
+        return {'messages': messages}
 
     def _get(self, req, project_id, queue_name):
         client_uuid = wsgi_helpers.get_client_uuid(req)
@@ -96,32 +112,38 @@ class CollectionResource(object):
             LOG.debug(ex)
             raise wsgi_errors.HTTPBadRequestAPI(six.text_type(ex))
 
-        except storage_errors.DoesNotExist as ex:
+        except storage_errors.QueueDoesNotExist as ex:
             LOG.debug(ex)
-            raise wsgi_errors.HTTPNotFound(six.text_type(ex))
+            messages = None
 
-        except Exception as ex:
-            LOG.exception(ex)
+        except Exception:
             description = _(u'Messages could not be listed.')
+            LOG.exception(description)
             raise wsgi_errors.HTTPServiceUnavailable(description)
 
         if not messages:
-            return None
+            messages = []
 
-        # Found some messages, so prepare the response
-        kwargs['marker'] = next(results)
-        base_path = req.path.rsplit('/', 1)[0]
-        messages = [wsgi_utils.format_message_v1(
-            m, base_path) for m in messages]
+        else:
+            # Found some messages, so prepare the response
+            kwargs['marker'] = next(results)
+            base_path = req.path.rsplit('/', 1)[0]
+            messages = [wsgi_utils.format_message_v1_1(m, base_path,
+                                                       m['claim_id'])
+                        for m in messages]
 
-        return {
-            'messages': messages,
-            'links': [
+        links = []
+        if messages:
+            links = [
                 {
                     'rel': 'next',
                     'href': req.path + falcon.to_query_str(kwargs)
                 }
             ]
+
+        return {
+            'messages': messages,
+            'links': links
         }
 
     # ----------------------------------------------------------------------
@@ -139,14 +161,23 @@ class CollectionResource(object):
             LOG.debug(ex)
             raise wsgi_errors.HTTPBadRequestAPI(six.text_type(ex))
 
-        # Deserialize and validate the request body
+        # Deserialize and validate the incoming messages
         document = wsgi_utils.deserialize(req.stream, req.content_length)
-        messages = wsgi_utils.sanitize(document, MESSAGE_POST_SPEC,
+
+        if 'messages' not in document:
+            description = _(u'No messages were found in the request body.')
+            raise wsgi_errors.HTTPBadRequestAPI(description)
+
+        messages = wsgi_utils.sanitize(document['messages'],
+                                       self._message_post_spec,
                                        doctype=wsgi_utils.JSONArray)
 
         try:
             self._validate.message_posting(messages)
 
+            if not self._queue_controller.exists(queue_name, project_id):
+                self._queue_controller.create(queue_name, project=project_id)
+
             message_ids = self._message_controller.post(
                 queue_name,
                 messages=messages,
@@ -161,14 +192,14 @@ class CollectionResource(object):
             LOG.debug(ex)
             raise wsgi_errors.HTTPNotFound(six.text_type(ex))
 
-        except storage_errors.MessageConflict as ex:
-            LOG.exception(ex)
+        except storage_errors.MessageConflict:
             description = _(u'No messages could be enqueued.')
+            LOG.exception(description)
             raise wsgi_errors.HTTPServiceUnavailable(description)
 
-        except Exception as ex:
-            LOG.exception(ex)
+        except Exception:
             description = _(u'Messages could not be enqueued.')
+            LOG.exception(description)
             raise wsgi_errors.HTTPServiceUnavailable(description)
 
         # Prepare the response
@@ -176,58 +207,92 @@ class CollectionResource(object):
         resp.location = req.path + '?ids=' + ids_value
 
         hrefs = [req.path + '/' + id for id in message_ids]
-
-        # NOTE(kgriffs): As of the Icehouse release, drivers are
-        # no longer allowed to enqueue a subset of the messages
-        # submitted by the client; it's all or nothing. Therefore,
-        # 'partial' is now always False in the v1.0 API, and the
-        # field has been removed in v1.1.
-        body = {'resources': hrefs, 'partial': False}
-
+        body = {'resources': hrefs}
         resp.body = utils.to_json(body)
         resp.status = falcon.HTTP_201
 
     @decorators.TransportLog("Messages collection")
     def on_get(self, req, resp, project_id, queue_name):
-        resp.content_location = req.relative_uri
-
         ids = req.get_param_as_list('ids')
+
         if ids is None:
             response = self._get(req, project_id, queue_name)
+
         else:
             response = self._get_by_id(req.path.rsplit('/', 1)[0], project_id,
                                        queue_name, ids)
 
         if response is None:
-            resp.status = falcon.HTTP_204
-            return
+            # NOTE(TheSriram): Trying to get a message by id, should
+            # return the message if its present, otherwise a 404 since
+            # the message might have been deleted.
+            msg = _(u'No messages with IDs: {ids} found in the queue {queue} '
+                    u'for project {project}.')
+            description = msg.format(queue=queue_name, project=project_id,
+                                     ids=ids)
+            raise wsgi_errors.HTTPNotFound(description)
 
-        resp.body = utils.to_json(response)
+        else:
+            resp.body = utils.to_json(response)
         # status defaults to 200
 
     @decorators.TransportLog("Messages collection")
     def on_delete(self, req, resp, project_id, queue_name):
-        # NOTE(zyuan): Attempt to delete the whole message collection
-        # (without an "ids" parameter) is not allowed
-        ids = req.get_param_as_list('ids', required=True)
-
+        ids = req.get_param_as_list('ids')
+        pop_limit = req.get_param_as_int('pop')
         try:
-            self._validate.message_listing(limit=len(ids))
-            self._message_controller.bulk_delete(
-                queue_name,
-                message_ids=ids,
-                project=project_id)
+            self._validate.message_deletion(ids, pop_limit)
 
         except validation.ValidationFailed as ex:
             LOG.debug(ex)
             raise wsgi_errors.HTTPBadRequestAPI(six.text_type(ex))
 
-        except Exception as ex:
-            LOG.exception(ex)
+        if ids:
+            resp.status = self._delete_messages_by_id(queue_name, ids,
+                                                      project_id)
+
+        elif pop_limit:
+            resp.status, resp.body = self._pop_messages(queue_name,
+                                                        project_id,
+                                                        pop_limit)
+
+    def _delete_messages_by_id(self, queue_name, ids, project_id):
+        try:
+            self._message_controller.bulk_delete(
+                queue_name,
+                message_ids=ids,
+                project=project_id)
+
+        except Exception:
             description = _(u'Messages could not be deleted.')
+            LOG.exception(description)
             raise wsgi_errors.HTTPServiceUnavailable(description)
 
-        resp.status = falcon.HTTP_204
+        return falcon.HTTP_204
+
+    def _pop_messages(self, queue_name, project_id, pop_limit):
+        try:
+            LOG.debug(u'POP messages - queue: %(queue)s, '
+                      u'project: %(project)s',
+                      {'queue': queue_name, 'project': project_id})
+
+            messages = self._message_controller.pop(
+                queue_name,
+                project=project_id,
+                limit=pop_limit)
+
+        except Exception:
+            description = _(u'Messages could not be popped.')
+            LOG.exception(description)
+            raise wsgi_errors.HTTPServiceUnavailable(description)
+
+        # Prepare response
+        if not messages:
+            messages = []
+        body = {'messages': messages}
+        body = utils.to_json(body)
+
+        return falcon.HTTP_200, body
 
 
 class ItemResource(object):
@@ -249,14 +314,17 @@ class ItemResource(object):
             LOG.debug(ex)
             raise wsgi_errors.HTTPNotFound(six.text_type(ex))
 
-        except Exception as ex:
-            LOG.exception(ex)
+        except Exception:
             description = _(u'Message could not be retrieved.')
+            LOG.exception(description)
             raise wsgi_errors.HTTPServiceUnavailable(description)
 
-        resp.content_location = req.relative_uri
-        message = wsgi_utils.format_message_v1(
-            message, req.path.rsplit('/', 2)[0])
+        # Prepare response
+        message['href'] = req.path
+        message = wsgi_utils.format_message_v1_1(message,
+                                                 req.path.rsplit('/', 2)[0],
+                                                 message['claim_id'])
+
         resp.body = utils.to_json(message)
         # status defaults to 200
 
@@ -289,9 +357,9 @@ class ItemResource(object):
                             u'deleted without a valid claim ID.')
             raise falcon.HTTPForbidden(error_title, description)
 
-        except Exception as ex:
-            LOG.exception(ex)
+        except Exception:
             description = _(u'Message could not be deleted.')
+            LOG.exception(description)
             raise wsgi_errors.HTTPServiceUnavailable(description)
 
         # Alles guete
