@@ -16,10 +16,8 @@
 from oslo_log import log as logging
 import six
 
-from zaqar.common import decorators
 from zaqar.i18n import _
 from zaqar.storage import errors as storage_errors
-from zaqar.transport import acl
 from zaqar.transport import utils
 from zaqar.transport.wsgi import errors as wsgi_errors
 
@@ -34,8 +32,6 @@ class Resource(object):
     def __init__(self, queue_controller):
         self._queue_ctrl = queue_controller
 
-    @decorators.TransportLog("Queues stats item")
-    @acl.enforce("queues:stats")
     def on_get(self, req, resp, project_id, queue_name):
         try:
             resp_dict = self._queue_ctrl.stats(queue_name,
@@ -72,7 +68,7 @@ class Resource(object):
             LOG.debug(ex)
             raise wsgi_errors.HTTPNotFound(six.text_type(ex))
 
-        except Exception as ex:
-            LOG.exception(ex)
+        except Exception:
             description = _(u'Queue stats could not be read.')
+            LOG.exception(description)
             raise wsgi_errors.HTTPServiceUnavailable(description)
