@@ -20,6 +20,7 @@ import six
 from zaqar.common import decorators
 from zaqar.i18n import _
 from zaqar.storage import errors as storage_errors
+from zaqar.transport import acl
 from zaqar.transport import utils
 from zaqar.transport import validation
 from zaqar.transport.wsgi import errors as wsgi_errors
@@ -27,35 +28,56 @@ from zaqar.transport.wsgi import utils as wsgi_utils
 
 LOG = logging.getLogger(__name__)
 
-CLAIM_POST_SPEC = (('ttl', int, None), ('grace', int, None))
-CLAIM_PATCH_SPEC = (('ttl', int, None), ('grace', int, 0))
 
+class CollectionResource(object):
+    __slots__ = (
+        '_claim_controller',
+        '_validate',
+        '_claim_post_spec',
+        '_default_meta',
+    )
 
-class Resource(object):
+    def __init__(self, wsgi_conf, validate, claim_controller,
+                 default_claim_ttl, default_grace_ttl):
 
-    __slots__ = ('_claim_controller', '_validate')
-
-    def __init__(self, wsgi_conf, validate, claim_controller):
         self._claim_controller = claim_controller
         self._validate = validate
 
+        self._claim_post_spec = (
+            ('ttl', int, default_claim_ttl),
+            ('grace', int, default_grace_ttl),
+        )
 
-class CollectionResource(Resource):
+        # NOTE(kgriffs): Create this once up front, rather than creating
+        # a new dict every time, for the sake of performance.
+        self._default_meta = {
+            'ttl': default_claim_ttl,
+            'grace': default_grace_ttl,
+        }
 
     @decorators.TransportLog("Claims collection")
+    @acl.enforce("claims:create")
     def on_post(self, req, resp, project_id, queue_name):
         # Check for an explicit limit on the # of messages to claim
         limit = req.get_param_as_int('limit')
         claim_options = {} if limit is None else {'limit': limit}
 
-        # Read claim metadata (e.g., TTL) and raise appropriate
-        # HTTP errors as needed.
-        document = wsgi_utils.deserialize(req.stream, req.content_length)
-        metadata = wsgi_utils.sanitize(document, CLAIM_POST_SPEC)
+        # NOTE(kgriffs): Clients may or may not actually include the
+        # Content-Length header when the body is empty; the following
+        # check works for both 0 and None.
+        if not req.content_length:
+            # No values given, so use defaults
+            metadata = self._default_meta
+        else:
+            # Read claim metadata (e.g., TTL) and raise appropriate
+            # HTTP errors as needed.
+            document = wsgi_utils.deserialize(req.stream, req.content_length)
+            metadata = wsgi_utils.sanitize(document, self._claim_post_spec)
 
         # Claim some messages
         try:
             self._validate.claim_creation(metadata, limit=limit)
+
             cid, msgs = self._claim_controller.create(
                 queue_name,
                 metadata=metadata,
@@ -70,33 +92,41 @@ class CollectionResource(Resource):
             LOG.debug(ex)
             raise wsgi_errors.HTTPBadRequestAPI(six.text_type(ex))
 
-        except Exception as ex:
-            LOG.exception(ex)
+        except Exception:
             description = _(u'Claim could not be created.')
+            LOG.exception(description)
             raise wsgi_errors.HTTPServiceUnavailable(description)
 
         # Serialize claimed messages, if any. This logic assumes
         # the storage driver returned well-formed messages.
         if len(resp_msgs) != 0:
-            resp_msgs = [wsgi_utils.format_message_v1(
-                msg, req.path.rpartition('/')[0], cid) for msg in resp_msgs]
+            base_path = req.path.rpartition('/')[0]
+            resp_msgs = [wsgi_utils.format_message_v1_1(msg, base_path, cid)
+                         for msg in resp_msgs]
 
             resp.location = req.path + '/' + cid
-            resp.body = utils.to_json(resp_msgs)
+            resp.body = utils.to_json({'messages': resp_msgs})
             resp.status = falcon.HTTP_201
         else:
             resp.status = falcon.HTTP_204
 
 
-class ItemResource(Resource):
+class ItemResource(object):
 
-    __slots__ = ('_claim_controller', '_validate')
+    __slots__ = ('_claim_controller', '_validate', '_claim_patch_spec')
 
-    def __init__(self, wsgi_conf, validate, claim_controller):
+    def __init__(self, wsgi_conf, validate, claim_controller,
+                 default_claim_ttl, default_grace_ttl):
         self._claim_controller = claim_controller
         self._validate = validate
 
-    @decorators.TransportLog("Claim item")
+        self._claim_patch_spec = (
+            ('ttl', int, default_claim_ttl),
+            ('grace', int, default_grace_ttl),
+        )
+
+    @decorators.TransportLog("Claims item")
+    @acl.enforce("claims:get")
     def on_get(self, req, resp, project_id, queue_name, claim_id):
         try:
             meta, msgs = self._claim_controller.get(
@@ -111,30 +141,31 @@ class ItemResource(Resource):
         except storage_errors.DoesNotExist as ex:
             LOG.debug(ex)
             raise wsgi_errors.HTTPNotFound(six.text_type(ex))
-        except Exception as ex:
-            LOG.exception(ex)
+        except Exception:
             description = _(u'Claim could not be queried.')
+            LOG.exception(description)
             raise wsgi_errors.HTTPServiceUnavailable(description)
 
         # Serialize claimed messages
         # TODO(kgriffs): Optimize
-        meta['messages'] = [wsgi_utils.format_message_v1(
-            msg, req.path.rsplit('/', 2)[0], meta['id'])
-            for msg in meta['messages']]
+        base_path = req.path.rsplit('/', 2)[0]
+        meta['messages'] = [wsgi_utils.format_message_v1_1(msg, base_path,
+                                                           claim_id)
+                            for msg in meta['messages']]
 
         meta['href'] = req.path
         del meta['id']
 
-        resp.content_location = req.relative_uri
         resp.body = utils.to_json(meta)
         # status defaults to 200
 
-    @decorators.TransportLog("Claim item")
+    @decorators.TransportLog("Claims item")
+    @acl.enforce("claims:update")
     def on_patch(self, req, resp, project_id, queue_name, claim_id):
         # Read claim metadata (e.g., TTL) and raise appropriate
         # HTTP errors as needed.
         document = wsgi_utils.deserialize(req.stream, req.content_length)
-        metadata = wsgi_utils.sanitize(document, CLAIM_PATCH_SPEC)
+        metadata = wsgi_utils.sanitize(document, self._claim_patch_spec)
 
         try:
             self._validate.claim_updating(metadata)
@@ -153,12 +184,13 @@ class ItemResource(Resource):
             LOG.debug(ex)
             raise wsgi_errors.HTTPNotFound(six.text_type(ex))
 
-        except Exception as ex:
-            LOG.exception(ex)
+        except Exception:
             description = _(u'Claim could not be updated.')
+            LOG.exception(description)
             raise wsgi_errors.HTTPServiceUnavailable(description)
 
-    @decorators.TransportLog("Claim item")
+    @decorators.TransportLog("Claims item")
+    @acl.enforce("claims:delete")
     def on_delete(self, req, resp, project_id, queue_name, claim_id):
         try:
             self._claim_controller.delete(queue_name,
@@ -167,7 +199,7 @@ class ItemResource(Resource):
 
             resp.status = falcon.HTTP_204
 
-        except Exception as ex:
-            LOG.exception(ex)
+        except Exception:
             description = _(u'Claim could not be deleted.')
+            LOG.exception(description)
             raise wsgi_errors.HTTPServiceUnavailable(description)
