@@ -42,11 +42,14 @@ from oslo_log import log
 import six
 
 from zaqar.common.api.schemas import pools as schema
+from zaqar.common import decorators
 from zaqar.common import utils as common_utils
 from zaqar.i18n import _
 from zaqar.storage import errors
 from zaqar.storage import utils as storage_utils
+from zaqar.transport import acl
 from zaqar.transport import utils as transport_utils
+from zaqar.transport import validation
 from zaqar.transport.wsgi import errors as wsgi_errors
 from zaqar.transport.wsgi import utils as wsgi_utils
 
@@ -59,9 +62,12 @@ class Listing(object):
     :param pools_controller: means to interact with storage
     """
 
-    def __init__(self, pools_controller):
+    def __init__(self, pools_controller, validate):
         self._ctrl = pools_controller
+        self._validate = validate
 
+    @decorators.TransportLog("Pools collection")
+    @acl.enforce("pools:get_all")
     def on_get(self, request, response, project_id):
         """Returns a pool listing as objects embedded in an object:
 
@@ -87,6 +93,12 @@ class Listing(object):
         request.get_param_as_int('limit', store=store)
         request.get_param_as_bool('detailed', store=store)
 
+        try:
+            self._validate.pool_listing(**store)
+        except validation.ValidationFailed as ex:
+            LOG.debug(ex)
+            raise wsgi_errors.HTTPBadRequestAPI(six.text_type(ex))
+
         cursor = self._ctrl.list(**store)
         pools = list(next(cursor))
 
@@ -124,11 +136,13 @@ class Resource(object):
         self._validators = {
             'weight': validator_type(schema.patch_weight),
             'uri': validator_type(schema.patch_uri),
-            'group': validator_type(schema.patch_uri),
+            'flavor': validator_type(schema.patch_flavor),
             'options': validator_type(schema.patch_options),
             'create': validator_type(schema.create)
         }
 
+    @decorators.TransportLog("Pools item")
+    @acl.enforce("pools:get")
     def on_get(self, request, response, project_id, pool):
         """Returns a JSON object for a single pool entry:
 
@@ -154,6 +168,8 @@ class Resource(object):
 
         response.body = transport_utils.to_json(data)
 
+    @decorators.TransportLog("Pools item")
+    @acl.enforce("pools:create")
     def on_put(self, request, response, project_id, pool):
         """Registers a new pool. Expects the following input:
 
@@ -178,17 +194,20 @@ class Resource(object):
         try:
             self._ctrl.create(pool, weight=data['weight'],
                               uri=data['uri'],
+                              flavor=data.get('flavor', None),
                               options=data.get('options', {}))
             response.status = falcon.HTTP_201
             response.location = request.path
         except errors.PoolCapabilitiesMismatch as e:
-            LOG.exception(e)
             title = _(u'Unable to create pool')
+            LOG.exception(title)
             raise falcon.HTTPBadRequest(title, six.text_type(e))
         except errors.PoolAlreadyExists as e:
-            LOG.exception(e)
+            LOG.exception('Pool "%s" already exists', pool)
             raise wsgi_errors.HTTPConflict(six.text_type(e))
 
+    @decorators.TransportLog("Pools item")
+    @acl.enforce("pools:delete")
     def on_delete(self, request, response, project_id, pool):
         """Deregisters a pool.
 
@@ -200,22 +219,24 @@ class Resource(object):
         try:
             self._ctrl.delete(pool)
         except errors.PoolInUseByFlavor as ex:
-            LOG.exception(ex)
             title = _(u'Unable to delete')
             description = _(u'This pool is used by flavors {flavor}; '
                             u'It cannot be deleted.')
             description = description.format(flavor=ex.flavor)
+            LOG.exception(description)
             raise falcon.HTTPForbidden(title, description)
 
         response.status = falcon.HTTP_204
 
+    @decorators.TransportLog("Pools item")
+    @acl.enforce("pools:update")
     def on_patch(self, request, response, project_id, pool):
         """Allows one to update a pool's weight, uri, and/or options.
 
         This method expects the user to submit a JSON object
-        containing at least one of: 'uri', 'weight', 'group', 'options'. If
-        none are found, the request is flagged as bad. There is also
-        strict format checking through the use of
+        containing at least one of: 'uri', 'weight', 'flavor',
+        'options'.If none are found, the request is flagged as bad.
+        There is also strict format checking through the use of
         jsonschema. Appropriate errors are returned in each case for
         badly formatted input.
 
@@ -225,11 +246,12 @@ class Resource(object):
         LOG.debug(u'PATCH pool - name: %s', pool)
         data = wsgi_utils.load(request)
 
-        EXPECT = ('weight', 'uri', 'options')
+        EXPECT = ('weight', 'uri', 'flavor', 'options')
         if not any([(field in data) for field in EXPECT]):
             LOG.debug(u'PATCH pool, bad params')
             raise wsgi_errors.HTTPBadRequestBody(
-                'One of `uri`, `weight`,or `options` needs '
+                'One of `uri`, `weight`, `flavor`,'
+                ' or `options` needs '
                 'to be specified'
             )
 
@@ -244,9 +266,13 @@ class Resource(object):
             )
         fields = common_utils.fields(data, EXPECT,
                                      pred=lambda v: v is not None)
-
+        resp_data = None
         try:
             self._ctrl.update(pool, **fields)
+            resp_data = self._ctrl.get(pool, False)
         except errors.PoolDoesNotExist as ex:
-            LOG.exception(ex)
+            LOG.exception('Pool "%s" does not exist', pool)
             raise wsgi_errors.HTTPNotFound(six.text_type(ex))
+
+        resp_data['href'] = request.path
+        response.body = transport_utils.to_json(resp_data)
